/*******************************************************************/
/*  Adaptation of BitBlt Painting To Borland TWindow Class         */
/*  Taken from Jitter.Doc  Carol P. Palkovic &David Thielen        */
/*  Adapted by Steven C. Leach 25 June 1993                        */
/*  This code is placed in the Public Domain                       */
/*  and may be used for any purpose the user/individual so desires */
/*                                                                 */
/*  The only restrictions I place on the individual/corporation    */
/*  that uses this code is to keep this header intact and          */
/*  to send me an an E-Mail 73577,175 about any changes they can.  */
/*                                                                 */
/*  I found this to be very helpful if rather memory intensive.    */
/*  Steven C. Leach , 639 Barto St. Santa Clara , CA 95051         */
/*  CIS 73577,175                                                  */
/*******************************************************************/

#include	"bltwin.hpp"



TBitBltWin::TBitBltWin(PTWindowsObject AParent, LPSTR ATitle)
                         : TWindow(AParent, ATitle)
{

	hBitmapDC = NULL;			// The bitmap DC we create
	hBitMap = NULL;			// The bitmap we create
	SetRectEmpty(&Rect);

}



// Define TMainWindow destructor
TBitBltWin::~TBitBltWin()
{
	if(hBitmapDC)
		DeleteDC (hBitmapDC);

	if(hBitMap)
		DeleteObject (hBitMap);

}

/* Response method for an incoming WM_PAINT message. Calls Paint,
  performing Windows-required paint setup and cleanup before and after.
  (If the TWindow has a TScroller, also calls its BeginView and EndView
  methods before and after call to Paint. */
/* Taken Directly from OWL Source Code Provided by Borland Inc.
	Praise Be to them for having the foresight to allow us to expand
	the functionality of their class
	*/
void TBitBltWin::WMPaint(RTMessage Msg)
{
	PAINTSTRUCT PaintInfo;

  if ( IsFlagSet(WB_ALIAS) ) // use application-defined wndproc
    DefWndProc(Msg);
  else
  {
    BltBeginPaint(HWindow, &PaintInfo);
    if ( Scroller )
      Scroller->BeginView(PaintInfo.hdc, PaintInfo);
    Paint(hBitmapDC, PaintInfo);
    if ( Scroller )
      Scroller->EndView();
    BltEndPaint(HWindow, &PaintInfo);
  }
}

void	TBitBltWin::WMEraseBkGnd(RTMessage Msg)
{
	// 0 means it is NOT erased!!!
	Msg.Result = 0L;
}

HDC     TBitBltWin::BltBeginPaint(HWND HWindow,PAINTSTRUCT FAR* pPaintStruct)
{
	DWORD dw;
	HDC hPaintDC;
	HBRUSH hBr;

	// We want to invalidate the entire rectangle.  Otherwise,
	// while we will paint the entire rectangle to the bitmap,
	// we will only blt the region - which is slower.
	GetUpdateRect (HWindow, &Rect, FALSE);
	InvalidateRect (HWindow, &Rect, FALSE);

	pSize.x = Rect.right - Rect.left;
	pSize.y = Rect.bottom - Rect.top;

	hPaintDC = BeginPaint (HWindow, pPaintStruct);

	// Create a DC with a bitmap to match the invalid screen rect
	hBitmapDC = CreateCompatibleDC (hPaintDC);
	hBitMap = CreateCompatibleBitmap (hPaintDC, pSize.x, pSize.y);
	SelectObject (hBitmapDC, hBitMap);

	// Set the viewport origin so it acts JUST like the window DC
	dw = GetViewportOrg (hBitmapDC);
	iXorg = LOWORD (dw);
	iYorg = HIWORD (dw);
	SetViewportOrg (hBitmapDC, iXorg - Rect.left,
				iYorg - Rect.top);

	// If this flag is set, we need to first init the bitmap to the
	// background brush.  We MUST get the brush aligned!
	if (pPaintStruct->fErase)

// I Changed this Since this extra word does not seem to be set in the 
// TWindow Class by Default and I don't want to perform any extra work if I 
// Don't have to 
//		if ((hBr = (HBRUSH) GetClassWord (HWindow, GCW_HBRBACKGROUND)) != NULL)
	if ((hBr = (HBRUSH) CreateSolidBrush((COLORREF)GetSysColor(COLOR_WINDOW))) != NULL)
			{
			UnrealizeObject (hBr);
			SetBrushOrg (hBitmapDC, (iXorg - Rect.left) & 7,
							(iYorg - Rect.top) & 7);
			FillRect (hBitmapDC, &Rect, hBr);
			}

	// Save hWnd & hDcBm for PaintEnd
	return (hBitmapDC);

}

void    TBitBltWin::BltEndPaint(HWND HWindow, const PAINTSTRUCT FAR* pPaintStruct)
{

	// Set back to text mode (in case changed).
	// Set the viewport org back to match the window
	SetMapMode (pPaintStruct->hdc, MM_TEXT);
	SetViewportOrg (pPaintStruct->hdc, iXorg, iXorg);
	SetMapMode (hBitmapDC, MM_TEXT);
	SetViewportOrg (hBitmapDC, iXorg, iXorg);

	// Copy the bitmap to the window
	BitBlt (pPaintStruct->hdc, Rect.left,Rect.top,
				pSize.x, pSize.y, hBitmapDC, 0, 0, SRCCOPY);

	// We're done - clean up
	DeleteDC (hBitmapDC);
	DeleteObject (hBitMap);
	EndPaint (HWindow, pPaintStruct);

	hBitmapDC = (HDC)NULL;
	hBitMap = (HBITMAP)NULL;
	SetRectEmpty(&Rect);

}

// These Function are here for completeness Sake
// I Personally never have to use them for my 
// implmentations.  Maybe I am Just Lucky.
DWORD    TBitBltWin::BltSetViewportOrg(int iXOrg,int iYOrg)
{
	if(hBitmapDC)
	{
		DWORD dw = SetViewportOrg(hBitmapDC,iXOrg - Rect.left,
						iYOrg - Rect.top);

		// Return The Corrected ol Value
		return (MAKELONG(LOWORD(dw) + Rect.left,
				HIWORD(dw)+Rect.top));
	}
	return (0L);
}

DWORD    TBitBltWin::BltGetViewportOrg(void)
{
	if(hBitmapDC)
	{
		DWORD dw = GetViewportOrg(hBitmapDC);
		
		// Return the Corrected Value
		return (MAKELONG (LOWORD (dw) + Rect.left,
				HIWORD(dw) + Rect.top));
	}
	return(0L);
}

