
#include "windows.h"
#include "paint.h"


// Call this function instead of BeginPaint

HDC PaintBegin (HWND hWnd, PAINTBLT *pPb)
{
DWORD dw;
HDC hDc, hDcBm;
HBRUSH hBr;

	// We want to invalidate the entire rectangle.  Otherwise,
	// while we will paint the entire rectangle to the bitmap,
	// we will only blt the region - which is slower.
	GetUpdateRect (hWnd, &(pPb->Rect), FALSE);
	InvalidateRect (hWnd, &(pPb->Rect), FALSE);

	pPb->pSize.x = pPb->Rect.right - pPb->Rect.left;
	pPb->pSize.y = pPb->Rect.bottom - pPb->Rect.top;

	hDc = BeginPaint (hWnd, &(pPb->sPs));

	// Create a DC with a bitmap to match the invalid screen rect
	hDcBm = CreateCompatibleDC (hDc);
	pPb->hBm = CreateCompatibleBitmap (hDc, pPb->pSize.x, pPb->pSize.y);
	SelectObject (hDcBm, pPb->hBm);

	// Set the viewport origin so it acts JUST like the window DC
	dw = GetViewportOrg (hDcBm);
	pPb->iXorg = LOWORD (dw);
	pPb->iYorg = HIWORD (dw);
	SetViewportOrg (hDcBm, pPb->iXorg - pPb->Rect.left,
																				pPb->iYorg - pPb->Rect.top);

	// If this flag is set, we need to first init the bitmap to the
	// background brush.  We MUST get the brush aligned!
	if (pPb->sPs.fErase)
		if ((hBr = (HBRUSH) GetClassWord (hWnd, GCW_HBRBACKGROUND)) != NULL)
			{
			UnrealizeObject (hBr);
			SetBrushOrg (hDcBm, (pPb->iXorg - pPb->Rect.left) & 7,
																	(pPb->iYorg - pPb->Rect.top) & 7);
			FillRect (hDcBm, &(pPb->Rect), hBr);
			}

	// Save hWnd & hDcBm for PaintEnd
	pPb->hWnd = hWnd;
	return (pPb->hDc = hDcBm);
}


// Call instead of EndPaint

void PaintEnd (PAINTBLT *pPb)
{
BITMAP bm;

	// Set back to text mode (in case changed).
	// Set the viewport org back to match the window
	SetMapMode (pPb->sPs.hdc, MM_TEXT);
	SetViewportOrg (pPb->sPs.hdc, pPb->iXorg, pPb->iXorg);
	SetMapMode (pPb->hDc, MM_TEXT);
	SetViewportOrg (pPb->hDc, pPb->iXorg, pPb->iXorg);

	// Copy the bitmap to the window
	BitBlt (pPb->sPs.hdc, pPb->Rect.left, pPb->Rect.top,
								pPb->pSize.x, pPb->pSize.y, pPb->hDc, 0, 0, SRCCOPY);

	// We're done - clean up
	DeleteDC (pPb->hDc);
	DeleteObject (pPb->hBm);
	EndPaint (pPb->hWnd, &(pPb->sPs));
}


// Call instead of SetViewportOrg

DWORD PaintSetViewOrg (PAINTBLT *pPb, int iXorg, int iYorg)
{
DWORD dw;

	// Set to the new value
	dw = SetViewportOrg (pPb->hDc, iXorg - pPb->Rect.left,
																					iYorg - pPb->Rect.top);

	// Return the corrected old value
	return (MAKELONG (LOWORD (dw) + pPb->Rect.left,
																	HIWORD (dw) + pPb->Rect.top));
}


// Call instead of GetViewportOrg

DWORD PaintGetViewOrg (PAINTBLT *pPb)
{
DWORD dw;

	dw = GetViewportOrg (pPb->hDc);

	// Return the corrected value
	return (MAKELONG (LOWORD (dw) + pPb->Rect.left,
																	HIWORD (dw) + pPb->Rect.top));
}


// Call to get the actual client area DC

HDC PaintGetDc (PAINTBLT *pPb)
{

	return (pPb->sPs.hdc);
}
